"""add_notes_to_accounting_entry

Revision ID: ba9718840e75
Revises: 82f27c891782
Create Date: 2025-05-29 08:47:20.597176

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = 'ba9718840e75'
down_revision: Union[str, None] = '82f27c891782'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('accounting_entries', schema=None) as batch_op:
        batch_op.add_column(sa.Column('notes', sa.String(), nullable=True))

    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('accounting_entries', schema=None) as batch_op:
        batch_op.drop_column('notes')

    # ### end Alembic commands ###
